#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create almost crazy effects of an image
#     (using all G'MIC artistic filters, one at a time obviously...)
#      the "craziness" is caused by applying the G'MIC filter using DIVIDE layer mode

# (c) Dinasset 2023
#
#   History:
#
#   v 1.1.0: 2023-08: Initial version
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys, os
import time
import datetime
import random
from random import randint
from gimpfu import *

# define here some constant value, when required
maxwh=5000

############################################################################
# Service routines
#---------------------------------------------------------------------------
# from Original drawable create the Output image
#---------------------------------------------------------------------------
def createOutimage(inImage, inLayer, widthheight, prepstep):
     
    owidth = inImage.width
    oheight = inImage.height
    otype = inImage.base_type
    oname = inLayer.name
    
   # define/calculate the scale factor for the (new) output image -------------------------
    realWidthHeight = owidth+oheight      # real size of the input image
    if widthheight == 0 :   # means the user wants to have output size same as input size
        if realWidthHeight<=maxwh:
            widthheight = realWidthHeight
        else:
            widthheight = maxwh  # max
            gimp.message ("forced max width+height")
    optiWidthHeight = widthheight       # size to work on to get best results
    scale = (optiWidthHeight*1.0)/(realWidthHeight*1.0)      # ratio for scaling

    rwidth=int(owidth*scale)
    rheight=int(oheight*scale)
    
   # create a new image of the decided size ------------------------------------------------
   # --- copy the source layer
   # --- scale
    newImage = pdb.gimp_image_new (rwidth, rheight, otype)                    
    newBG = pdb.gimp_layer_new_from_drawable(inLayer, newImage)     # background layer
    newBG.mode = LAYER_MODE_NORMAL
    newBG.name = "NewImage: Background Layer from "+oname
    newBG.opacity = 100.0 
    newImage.add_layer(newBG, 0)
    pdb.gimp_layer_scale(newBG, rwidth, rheight, False)
    pdb.gimp_layer_add_alpha (newBG)    # RGBA
    
   # keep a copy of the original background below the one which will be used ----------------
   # --- and set its visibility off (it should not enter in the process)
    Original = newBG.copy()
    newImage.add_layer(Original, 1)     # below
    pdb.gimp_item_set_visible (Original, False)
    Original.name = oname+" (original)"
    
    # create base for effect
    pdb.gimp_image_set_active_layer (newImage,  newBG)

    # optional user choice for preprocessing
    if prepstep>0:  #user did a selection
        #               #"DCP Dehaze (good for light image",
        #               #"Dynamic Range Increase",
        #               #"Equalize Local Histogram",
        #               #"Local Contrast Enhancement",
        #               #"Local Normalization",
        #               #"Local Processing (Equalize)",
        #               #"Local Processing (Normalize)",
        if prepstep==1: # jeje_dehaze 5,1,0.2,1,0,0,0,0,0
            parms='5,1,0.2,1,0,0,0,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -jeje_dehaze "+str(parms)+" ")
        elif prepstep==2: # fx_tk_dri 0.5,0.5,0.5,2.5,0.5,1,1,0
            parms='0.5,0.5,0.5,2.5,0.5,1,1,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_tk_dri "+str(parms)+" ")
        elif prepstep==3: # fx_equalize_local_histograms 75,2,4,100,8,1,16,0,50,50
            parms='75,2,4,100,8,1,16,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_equalize_local_histograms "+str(parms)+" ")
        elif prepstep==4: # fx_LCE 80,0.5,1,1,0,0
            parms='80,0.5,1,1,0,0'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_LCE "+str(parms)+" ")
        elif prepstep==5: # fx_normalize_local 2,6,5,20,1,11,0,50,50
            parms='2,6,5,20,1,11,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_normalize_local "+str(parms)+" ")
        elif prepstep==6: # fx_local_processing 1,75,10,50,20,0,7,0,50,50
            parms='1,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
        elif prepstep==7: # fx_local_processing 0,75,10,50,20,0,7,0,50,50
            parms='0,75,10,50,20,0,7,0,50,50'
            pdb.plug_in_gmic_qt(newImage,newBG, 1,0,
            "-v - -fx_local_processing "+str(parms)+" ")
    newBG = pdb.gimp_image_get_active_layer (newImage)
            
    # adjust mean value of the base
    # "normal" mean value should be betweeb 117.5 and 137.5
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG, HISTOGRAM_LUMINANCE, 0.0, 1.0)
    #gimp.message ("mean="+str(mean))
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (newBG,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        newBG.name = newBG.name+" mean adj"
    newBG.opacity = 100.0
    
    return(newImage)
#--------------------------------------------------------------------------- 
    

############################################################################
# Main
#---------------------------------------------------------------------------

def ArtisticDivide (inImage, inLayer,
                    widthheight, style, prepstep,
                    doFlatten):
    
    
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H-%M-%S")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    
    newImage = createOutimage (inImage, inLayer, widthheight, prepstep)
    baseforeffect = newImage.layers[0]
    baseforeffect.name = "base for effect"
    #pdb.gimp_display_new (newImage)
    #return
    
    # environment setups
    finallayermode = 0  # (*auto)
    autolayermode = LAYER_MODE_NORMAL   # (default)
    
    # apply chosen GMIC Artistic effect
    # ---------------------------------
    
    if style==0:     #Anguish
        # apply ANQUISH
        #[G'MIC] samj_Angoisse 1,5,0,5,100,2,4,1,250
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1,5,0,5,100,2,4,1,250'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Angoisse "+str(parms)+" ")
        StyleLayer.name = "Anguish Layer "+str(parms)
        StyleLayer.opacity = 3.0        
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)    
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean>200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE,
                                      0.0, 1.0, False,
                                      0.25, 0.1, 0.9, False)    
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE,
                                      0.0, 1.0, False,
                                      0.5, 0.1, 0.9, False)   
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT
         
    elif style==1:     #Aurora
        # apply AURORA
        #[G'MIC] gcd_aurora 6,1,1
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='6,1,1'   
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -gcd_aurora "+str(parms)+" ")
        StyleLayer.name = "Aurora Layer "+str(parms)
        StyleLayer.opacity = 10.0 
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Vibrance  
        pdb.plug_in_gmic_qt(newImage,Styled, 1,0,
        "-v - -fx_vibrance 2,0,50,50 ")  
        
        if finallayermode==0: autolayermode = LAYER_MODE_LINEAR_BURN       
                 
    elif style==2:    #Barbouillage
        # apply BARBOUILLAGE
        #[G'MIC] samj_Barbouillage_Paint_Daub 2,2,100,0.2,1,4,1,0,8
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='2,2,100,0.2,1,4,1,0,8'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Barbouillage_Paint_Daub "+str(parms)+" ")  
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "Barbouillage Layer "+str(parms)
        StyleLayer.opacity = 20.0    
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)    
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean>190:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE,
                                      0.0, 1.0, False,
                                      0.35, 0.1, 0.9, False)   
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE,
                                      0.0, 1.0, False,
                                      0.55, 0.1, 0.9, False)   
       # APPLY LEVELS was
        #pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
        #                                  0.75, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_HSV_VALUE    
        
    elif style==3:    #Crayongraffiti
        # apply BLACK CRAYON GRAFFITI
        #[G'MIC] fx_crayongraffiti2 2500,25,1,0.4,12,1,2,2,0 july 14
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='2500,25,1,0.4,12,1,2,2,0' # july 14
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_crayongraffiti2 "+str(parms)+" ")
        StyleLayer.name = "Graffiti Layer "+str(parms)
        StyleLayer.opacity = 30.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # APPLY LEVELS  
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  0.25, 0.1, 0.9, False)   
        pdb.gimp_drawable_desaturate (StyleLayer, False)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        #pdb.gimp_display_new (newImage)
        #return
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.2, 0.1, 0.9, False)  
        # Apply Vibrance  
        pdb.plug_in_gmic_qt(newImage,Styled, 1,0,
        "-v - -fx_vibrance 0.125,0,50,50")
        if finallayermode==0: autolayermode = LAYER_MODE_PIN_LIGHT     
             
    elif style==4:    #Blockism
        # apply BLOCKISM
        #[G'MIC] fx_blockism 4,1.6,0.5,50,0.5,64,0,0,0
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='4,1.6,0.5,50,0.5,64,0,0,0'   #new
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_blockism "+str(parms)+" ")
        StyleLayer.name = "Blockism Layer "+str(parms)
        StyleLayer.opacity = 15.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.7, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT    
    
    elif style==5:  #Bokeh
        # apply BOKEH
        #[G'MIC] fx_bokeh 3,8,0,30,8,4,0.5,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='3,8,0,30,8,4,0.5,0.2,210,210,80,160,0.7,30,20,20,1,2,170,130,20,110,0.15,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_bokeh "+str(parms)+" ")
        StyleLayer.name = "Bokeh Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # add desaturate  
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMINANCE)
        # APPLY LEVELS  
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.5, 0.1, 0.9, False)    
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # add plasma  
        pdb.plug_in_gmic_qt(newImage,Styled, 1,2,
        "-v - -fx_plasma 0.5,0,8,0,1,128,128,128 ") 
        Temp = pdb.gimp_image_get_active_layer (newImage)
        Styled = pdb.gimp_image_merge_down (newImage, Temp, CLIP_TO_BOTTOM_LAYER)
        
        #pdb.gimp_display_new (newImage)
        #return
    
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.5, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY    
     
    elif style==6:    #Brushify
        # apply BRUSHIFY
        #[G'MIC] fx_brushify 7,0.25,4,48,24,12,0,2,4,0.2,0.5,30,0.997,1,1,5,0,0.2,1     
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,  
                              StyleLayer.height/2,
                              True)
        parms='7,0.25,4,48,24,12,0,2,4,0.2,0.5,30,0.997,1,1,5,0,0.2,1'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_brushify "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)                 
        StyleLayer.name = "Brushify Layer "+str(parms)
        StyleLayer.opacity = 10.0    
        StyleLayer.mode = LAYER_MODE_DIVIDE
        
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        #pdb.gimp_display_new (newImage)
        #return
              
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.5, 0.1, 0.9, False)
        # add local contrast     
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -afre_localcontrast 1,50 ")
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY    
              
    elif style==7:    #Cartoon
        # apply CARTOON
        #[G'MIC] cartoon 3,200,20,0.25,1.5,8,0,50,50
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='3,200,20,0.25,1.5,8,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - cartoon "+str(parms)+" ")
        StyleLayer.name = "Cartoon Layer "+str(parms)
        StyleLayer.opacity = 55.0   
        StyleLayer.mode = LAYER_MODE_DIVIDE
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (StyleLayer, HISTOGRAM_VALUE,
                                                        0.0, 1.0)    
        #pdb.gimp_message ("mean StyleLayer="+str(mean))
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
                      
        # APPLY LEVELS  
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)   
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean>240:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.15, 0.1, 0.9, False)
        elif mean>220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.1, 0.9, False)
        elif mean>200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.45, 0.1, 0.9, False)
        elif mean>180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.66, 0.1, 0.9, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.1, 0.9, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY    
            
    elif style==8:    #ChalkItUp
        # apply CHALKITUP
        #[G'MIC] samj_chalkitup 5,2.5,1.5,50,1,5,5,0,0,7,0.8,1.9,7,0
        
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_chalkitup 5,2.5,1.5,50,1,5,5,0,0,7,0.8,1.9,7,0 ")
        StyleLayer.name = "Chalkitup Layer 5,2.5,1.5,50,1,5,5,0,0,7,0.8,1.9,7,0"
        StyleLayer.opacity = 25.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                                0.4, 0.1, 0.9, False)   
        if finallayermode==0: autolayermode = LAYER_MODE_LINEAR_BURN
        
    elif style==9:    #CircleAbstraction
        # apply CIRCLE ABSTRACTION
        #[G'MIC] fx_circle_abstraction 16,5,0.8,1,0,1,0,0,50,50
        
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='16,5,0.8,1,0,1,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_circle_abstraction "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "CircleAbstr Layer "+str(parms)
        StyleLayer.opacity = 35.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY
        
    elif style==10:    #ColorAbstraction
        # apply COLOR ABSTRACTION PAINT
        #[G'MIC] fx_ColorAbstractionPaint 10,5.05,1,0,1,1,1,1,1,1,0,0,0,0

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='10,5.05,1,0,1,1,1,1,1,1,0,0,0,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ColorAbstractionPaint "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "ColorAbstr Layer "+str(parms)
        StyleLayer.opacity = 3.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY
        
    elif style==11:    #Colored Pencils
        # apply COLORED PENCILS
        #[G'MIC] fx_cpencil 1.3,50,20,2,2,1,0

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1.3,50,20,2,2,1,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cpencil "+str(parms)+" ")
        StyleLayer.name = "ColorPencil Layer "+str(parms)
        StyleLayer.opacity = 100.0   #chg july 15 from 5.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # added special effect
        Temp = Styled.copy()
        newImage.add_layer (Temp, 0)
        pdb.gimp_drawable_equalize (Temp, False)
        pdb.gimp_drawable_invert (Temp, False)
        Temp.mode = LAYER_MODE_DARKEN_ONLY
        Styled = pdb.gimp_image_merge_down (newImage, Temp,
                                            CLIP_TO_BOTTOM_LAYER)
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.0, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_LINEAR_LIGHT   
        
    elif style==12:    #Comic Book
        # apply COMICBOOK
        #[G'MIC] cl_comic 7,2,0,1,2,15,15,1,0,30,6,2,0,0,0,0,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='7,2,0,1,2,15,15,1,0,30,6,2,0,0,0,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -cl_comic "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)           
        StyleLayer.name = "Comic Layer "+str(parms)
        StyleLayer.opacity = 30.0    
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # added special effect
        Temp = Styled.copy()
        newImage.add_layer (Temp, 0)
        pdb.gimp_drawable_equalize (Temp, False)
        pdb.gimp_drawable_invert (Temp, False)
        Temp.mode = LAYER_MODE_DARKEN_ONLY
        Styled = pdb.gimp_image_merge_down (newImage, Temp,
                                            CLIP_TO_BOTTOM_LAYER)
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                                1.0, 0.1, 0.9, False)   
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT   
        
    elif style==13:    #Cubism
        # apply CUBISM
        #[G'MIC] fx_cubism 2,50,10,90,0.7,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='2,50,10,90,0.7,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cubism "+str(parms)+" ")
        StyleLayer.name = "Cubism Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY
        
    elif style==14:    #Cutout
        # apply CUTOUT
        #[G'MIC] fx_cutout 15,3,0,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='15,3,0,1,0,50,50'  
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_cutout "+str(parms)+" ")
        StyleLayer.name = "Cutout Layer "+str(parms)
        pdb.gimp_drawable_equalize (StyleLayer, False)  # (False=no mask)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS (NO TEST NEEDED, mean always very light)
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE,
                                  0.0, 1.0, False,
                                  0.50, 0.0, 1.0, False)
        
        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
            
                                           
    elif style==15:    #Diffusion Tensors
        # apply DIFFUSION TENSORS
        #[G'MIC] fx_diffusiontensors 10,5,3,1,0.15,1,0,3,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='10,5,3,1,0.15,1,0,3,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_diffusiontensors "+str(parms)+" ")
        StyleLayer.name = "DTensors Layer "+str(parms)
        pdb.gimp_drawable_equalize (StyleLayer, False)  #  (False=no mask)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (StyleLayer, HISTOGRAM_VALUE,
                                                        0.0, 1.0)      
        #pdb.gimp_message ("mean StyleLayer="+str(mean))
        if mean<140:
            pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE,
                                      0.0, 1.0, False,
                                      1.75, 0.0, 1.0, False)    
            
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)    
        #pdb.gimp_message ("mean Styled="+str(mean))
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                                0.3, 0.1, 0.9, False)   
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT     
                                           
    elif style==16:    #Doodle
        # apply DOODLE
        #[G'MIC] fx_doodle 30,2,2,1.5,2,70,0    

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
         
        # reduce size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width*0.5,
                              StyleLayer.height*0.5,
                              True)
        # now Apply DOODLE
        parms='30,2,2,1.5,2,70,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_doodle "+str(parms)+" ")
         
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)
        # add Apply Threshold
        pdb.gimp_drawable_threshold (StyleLayer, HISTOGRAM_VALUE, 0.9, 1.0)
                               
        StyleLayer.name = "Doodle Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # add reduce extremes
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  1.0, 0.3, 0.7, False)
        
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # add colorize
        pdb.gimp_drawable_colorize_hsl (Styled, 180.0, 00.0, -30.0)

        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  0.5, 0.1, 0.9, False)   
        if finallayermode==0: autolayermode = LAYER_MODE_SCREEN
            
    #pdb.gimp_display_new (newImage)
    #return
                                           
    elif style==17:    #DreamSmoothing
        # apply DREAM SMOOTHING
        #[G'MIC] fx_dreamsmooth 3,1,2,1,1,2,1,24,0      2=and

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # reduce size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='3,1,2,1,1,2,1,24,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_dreamsmooth "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "DreamSmooth Layer "+str(parms)
        StyleLayer.opacity = 3.0  #50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean>200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.25, 0.1, 0.9, False)
        elif mean>170:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.50, 0.1, 0.9, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.75, 0.1, 0.9, False)
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT  
                                           
    elif style==18:    #Ellipsionism
        # apply ELLIPSIONISM
        #[G'MIC] fx_ellipsionism 20,10,0.5,0.7,3,0.5,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='20,10,0.5,0.7,3,0.5,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ellipsionism "+str(parms)+" ")
    
        StyleLayer.name = "Ellipsionism Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.15, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT   
        
    elif style==19:    #Felt Pen
        # apply FELT PEN
        #[G'MIC] fx_feltpen 300,12,0,0.1,0,5,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='300,12,0,0.1,0,5,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_feltpen "+str(parms)+" ")
    
        StyleLayer.name = "FeltPen Layer "+str(parms)
        StyleLayer.opacity = 100.0  
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        #pdb.gimp_display_new (newImage)
        #return
        
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean>200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.40, 0.0, 1.0, False)
        elif mean>170:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.50, 0.0, 1.0, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.65, 0.0, 1.0, False)
            
        if finallayermode==0: autolayermode = LAYER_MODE_PIN_LIGHT  
                                           
    elif style==20:    #FingerPaint
        # apply FINGER PAINT
        #[G'MIC] gtutor_fpaint 0.35,0.7,0,0,45,0.5,0.5,0.5,0   

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/3,
                              StyleLayer.height/3,
                              True)
        parms='0.35,0.7,0,0,45,0.5,0.5,0.5,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -gtutor_fpaint "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "FingerPaint Layer "+str(parms)
        StyleLayer.opacity = 5.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # APPLY LEVELS
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                              0.35, 0.1, 0.9, False)    
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.35, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_SCREEN  
        
    elif style==21:    #Fractalize
        # apply FRACTALIZE
        #[G'MIC] fractalize 0.7      

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
    
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fractalize 0.7 ")
    
        StyleLayer.name = "Fractal Layer 0.7"
        StyleLayer.opacity = 10.0        
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT
        
    elif style==22:    #Ghost
        # apply GHOST
        #[G'MIC] fx_ghost 200,2,2,1,3,16,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='200,2,2,1,3,16,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_ghost "+str(parms)+" ")
    
        StyleLayer.name = "Ghost Layer "+str(parms)
        StyleLayer.opacity = 25.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT
        
    elif style==23:    #Graphic Boost
        # apply GRAPHIC BOOST
        #[G'MIC] fx_graphic_boost4 1.25,2,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1.25,2,0,0.15,14,0,1,0.5,0.45,2,0,0,0,1,1,1,0.5,0.45,1'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_graphic_boost4 "+str(parms)+" ")
    
        StyleLayer.name = "GBoost Layer "+str(parms)
        StyleLayer.opacity = 5.0     
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.75, 0.1, 0.9, False)     
        if finallayermode==0: autolayermode = LAYER_MODE_HARDLIGHT
        
    elif style==24:    #Graphic Novel
        # apply GRAPHIC NOVEL
        #[G'MIC] fx_graphic_novelfxl 0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,10,1,1,1,0.5,0.8,1.28

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='0,2,6,5,20,0,0.62,14,0,1,0.5,0.78,1.92,0,0,10,1,1,1,0.5,0.8,1.28'  
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_graphic_novelfxl "+str(parms)+" ")
    
        StyleLayer.name = "GNovel Layer "+str(parms)
        StyleLayer.opacity = 5.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.75, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT
                                                                                                         
    #pdb.gimp_display_new (newImage)
    #return
        
    elif style==25:    #Hard Sketch
        # apply HARD SKETCH
        #[G'MIC] fx_hardsketchbw 300,30,6,0.3,0,0,3,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='300,30,6,0.3,0,0,3,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_hardsketchbw "+str(parms)+" ")
    
        StyleLayer.name = "HSketch Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
       #pdb.gimp_drawable_threshold (StyleLayer, HISTOGRAM_VALUE, 0.01, 1.0)
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.8, 0.1, 0.9, False)    
        if finallayermode==0: autolayermode = LAYER_MODE_DARKEN_ONLY
                                                                                                         
    #pdb.gimp_display_new (newImage)
    #return
    
    elif style==26:   #Highlight Bloom
        # Apply Style
        #[G'MIC] fx_highlight_bloom 90,60,60,30,20,0,50,50
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='90,60,60,30,20,0,50,50'  #new
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_highlight_bloom "+str(parms)+" ")
    
        StyleLayer.name = "HLBloom Layer "+str(parms)
        StyleLayer.opacity = 100.0   # new, was 10
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMINANCE) #new
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # APPLY LEVELS  
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        pdb.gimp_message ("mean Styled="+str(mean))
        if mean>220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.65, 0.0, 1.0, False)
        elif mean>180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.40, 0.0, 1.0, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.25, 0.0, 1.0, False)
            
        if finallayermode==0: autolayermode = LAYER_MODE_HARDLIGHT     
       
    elif style==27:    #Hope Poster
        # apply HOPE POSTER
        #[G'MIC] fx_poster_hope 0,3,0,50,50  

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='0,20,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_poster_hope "+str(parms)+" ")
    
        StyleLayer.name = "HPoster Layer "+str(parms)
        StyleLayer.opacity = 100.0 
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
         
        # APPLY LEVELS (NO TEST NEEDED, mean always very light)
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  0.50, 0.00, 1.00, False)
        
        if finallayermode==0: autolayermode = LAYER_MODE_LINEAR_BURN
            
    elif style==28:     #HoughSketch
        # Apply Style
        #[G'MIC] fx_houghsketchbw 1.25,10,4,90,0.08,3,0,50,50
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1.25,10,4,90,0.08,3,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_houghsketchbw "+str(parms)+" ")
    
        StyleLayer.name = "HPoster Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_display_new (newImage)
        #return       
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.00, 0.25, 0.75, False)    # ONLY INTERMEDUATE
        elif mean<195:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.75, 0.25, 0.75, False)    # ONLY INTERMEDUATE
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.50, 0.25, 0.75, False)    # ONLY INTERMEDUATE
        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
            
        
    if style==29:    #Illustration Look
        # Apply Style
        #[G'MIC] fx_illustration_look 20,0,0,0,0    
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # reduce size, because slow              
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width*2/3,
                              StyleLayer.height*2/3,
                              True)
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  0.5, 0.25, 0.75, False)  # only mid values  
        parms='20,0,0,0,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_illustration_look "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)         
        StyleLayer.name = "IllustrationL Layer "+str(parms)
        StyleLayer.opacity = 1.0   # was 100
        StyleLayer.mode = LAYER_MODE_DIVIDE
        
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # APPLY LEVELS
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.85, 0.10, 0.90, False)   
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT     
        
    elif style==30:    #Kuwahara
        # Apply Kuwahara
        #[G'MIC] fx_kuwahara 2,5,0,0,0,50,50
    
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        
        parms='2,5,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_kuwahara "+str(parms)+" ")
        
        StyleLayer.name = "Kuwahara Layer "+str(parms)
        StyleLayer.opacity = 15.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.5, 0.10, 0.90, False)   
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_GRAIN_MERGE     
    
    elif style==31:    #Line Art
        # apply Photo to Line Art
        #[G'MIC] cl_lineart 4,0,5,1,19,100,0,0,6,2,2,0,0,0,50,50
        
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # reduce size, because slow
        pdb.plug_in_median_blur (newImage, StyleLayer, 5, 0.0)
         
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width*5/6,
                              StyleLayer.height*5/6,
                              True)
        '''
        fill holes before
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fill_holes 11,21,5,0,0,1")
        ''' 
        #apply lineart now  
        parms='4,0,5,1,19,100,0,0,6,2,2,0,0,0,50,50'   
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -cl_lineart "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)
        StyleLayer.name = "LineArt Layer "+str(parms)
        StyleLayer.opacity = 100.0
        pdb.gimp_drawable_threshold (StyleLayer, HISTOGRAM_VALUE,
                                    0.75, 1.0)
        pdb.gimp_image_select_color (newImage, CHANNEL_OP_REPLACE,
                                     StyleLayer, (0.0,0.0,0.0,1.0))
        pdb.gimp_context_set_foreground ((0.0,0.0,0.35,1.0))                             
        pdb.gimp_edit_bucket_fill (StyleLayer, BUCKET_FILL_FG,
                                   LAYER_MODE_NORMAL, 100.0, 0.0, False,
                                   0.0, 0.0)
        pdb.gimp_selection_invert (newImage)
        pdb.gimp_context_set_foreground ((0.85,0.85,0.85,1.0))                             
        pdb.gimp_edit_bucket_fill (StyleLayer, BUCKET_FILL_FG,
                                   LAYER_MODE_NORMAL, 100.0, 0.0, False,
                                   0.0, 0.0)
        pdb.gimp_selection_none (newImage)
       
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_display_new (newImage)
        #return
         
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<185:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.75, 0.12, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.35, 0.16, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.20, 0.24, 0.90, False)
        if finallayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
                                           
    elif style==32:    #Linify
        # apply LINIFY
        #[G'MIC] fx_linify 40,2,40,10,24,0,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        parms='40,2,40,10,24,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_linify "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Linify Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.75, 0.16, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.24, 0.90, False)
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY
       
    elif style==33:    #Lyle Painting
        # apply Lyle Painting
        #[G'MIC] fx_lylejk_painting 10,10,4,50,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        #pdb.gimp_display_new (newImage)
        #return
        parms='10,10,4,50,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_lylejk_painting "+str(parms)+" ")
    
        StyleLayer.name = "LylePaint Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_equalize (StyleLayer, False)
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.5, 0.1, 0.9, False)  
        if finallayermode==0: autolayermode = LAYER_MODE_MULTIPLY
         
    elif style==34:    #Make Squiggly
        # apply Make Squiggly
        #[G'MIC] fx_Squiggly 20,15,5,0,0.99,1,0,0,3,0.2,0.4,0,1,1,0

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='20,15,5,0,0.99,1,0,0,3,0.2,0.4,0,1,1,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_Squiggly "+str(parms)+" ")
    
        StyleLayer.name = "Squiggly Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_display_new (newImage)
        #return
        pdb.gimp_drawable_brightness_contrast (StyleLayer, 0.0, 0.33)
        
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        if finallayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        
    elif style==35:    #Morphology Paint
        # apply Morphology Paint
        #[G'MIC] fx_MorphoPaint 1,18,2,25,100,230,8,3,4,0.5,2,0.5,200,1,1,1,1,1,1,1,1,0

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1,18,2,25,100,230,8,3,4,0.5,2,0.5,200,1,1,1,1,1,1,1,1,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_MorphoPaint "+str(parms)+" ")
    
        StyleLayer.name = "MorphoPaint Layer "+str(parms)
        StyleLayer.opacity = 80.0
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_VALUE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.55, 0.10, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.10, 0.90, False)
        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
        
    elif style==36:    #Paint with Brush
        # apply Paint with Brush
        #[G'MIC] fx_paint_with_brush 0,"0",1,16,1,100,100,10,80,0.5,3,45,0,6,2,10,0,0,0,60,20,1,0,0,0,30,15,15,15,15,15,15,1,45,0,0,50,50
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='0,"0",1,16,1,100,100,10,80,0.5,3,45,0,6,2,10,0,0,0,60,20,1,0,0,0,30,15,15,15,15,15,15,1,45,0,0,50,50,1,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_paint_with_brush "+str(parms)+" ")
    
        StyleLayer.name = "PaintwBrush Layer "+str(parms)
        StyleLayer.opacity = 60.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.65, 0.10, 0.85, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.10, 0.80, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.18, 0.10, 0.75, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
        
    elif style==37:    #Painting 
        # apply Painting
        #[G'MIC] fx_painting 10,5,0,0,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='10,5,0,0,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_painting "+str(parms)+" ")
    
        StyleLayer.name = "Painting Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  1.00, 0.50, 1.00, False)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<190:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.65, 0.10, 0.85, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.10, 0.80, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.18, 0.10, 0.75, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
        
    elif style==38:    #Pastel Art 
        # apply Pastel Art
        #[G'MIC] fx_pastell 0.6,1,0,20,30,300,1,30,1,1,0.5,0,0,10,3,0,9,3,0,12,0
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='0.6,1,0,20,30,300,1,30,1,1,0.5,0,0,10,3,0,9,3,0,12,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pastell "+str(parms)+" ")
    
        StyleLayer.name = "PastelArt Layer "+str(parms)
        StyleLayer.opacity = 100.0
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
        StyleLayer.mode = LAYER_MODE_DIVIDE
        # increase local contrast
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -afre_localcontrast 3,100 ")
        #
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        if mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<210:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.50, 0.10, 0.80, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.10, 0.70, False)

        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT     
        
    elif style==39:    #Pen Drawing 
        # apply Pen Drawing
        #[G'MIC] fx_pen_drawing 30,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        # increase local contrast
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -afre_localcontrast 3,100 ")
        
        parms='15,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_pen_drawing "+str(parms)+" ")
    
        StyleLayer.name = "PenDraw Layer "+str(parms)
        StyleLayer.opacity = 100.0
        pdb.gimp_drawable_levels_stretch (StyleLayer)
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
        #pdb.gimp_display_new (newImage)
        #return
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
          
        # increase local contrast
        pdb.plug_in_gmic_qt(newImage,Styled, 1,0,
        "-v - -afre_localcontrast 3,100 ")
          
        pdb.gimp_drawable_equalize (Styled, False)
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.50, 0.10, 0.90, False)

        if finallayermode==0: autolayermode = LAYER_MODE_SOFTLIGHT   
             
    elif style==40:    #Photo Illustration 
        # apply Photo Illustration
        #[G'MIC] fx_tk_photoillustration 0,0.25,0,0.3,0.5,0.5,1,0,5,0,1,0,0,1,0,0,1.2,98.5,5,0.5,0,0,0,0,0,1,0
        parms='0,0.25,0,0.3,0.5,0.5,1,0,5,0,1,0,0,1,0,0,1.2,98.5,5,0.5,0,0,0,0,0,1,0'
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        StyleLayer.name = "PhotoIll Layer "+str(parms)
         
        StyleLayer.opacity = 25.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.20, 0.80, False)
         
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_tk_photoillustration "+str(parms)+" ")
        # note: it seems that after the application the layer name changes
        # hence, attributes have been moved before application
        #StyleLayer.name = "PhotoIll Layer ..."
        #StyleLayer.opacity = ...
        #StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.50, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.35, 0.15, 0.85, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.20, 0.20, 0.80, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
             
    elif style==41:    #Poligonize Delauney 
        # apply Poligonize Delauney 
        #[G'MIC] fx_polygonize_delaunay 40,5,75,0.5,3,50,255,255,0,255,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='40,5,75,0.5,3,50,255,255,0,255,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_polygonize_delaunay "+str(parms)+" ")
        StyleLayer.name = "Polygonize Delauney Layer "+str(parms)
        StyleLayer.opacity = 60.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.60, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.36, 0.12, 0.85, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.15, 0.80, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY    
        
    elif style==42:    #Poligonize [Energy] 
        # apply Poligonize [Energy] 
        #[G'MIC] fx_polygonize 300,10,10,10,10,255,255,0,255,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='300,10,10,10,10,255,255,0,255,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_polygonize "+str(parms)+" ")
        StyleLayer.name = "Polygonize Energy Layer "+str(parms)
        StyleLayer.opacity = 30.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.60, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.36, 0.12, 0.85, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.15, 0.80, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
            
    elif style==43:    #Poster Edges 
        # apply Poster Edges 
        #[G'MIC] fx_poster_edges 20,60,5,0,10,8,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='20,60,5,0,10,8,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_poster_edges "+str(parms)+" ")
        StyleLayer.name = "PosterEdges Layer "+str(parms)
        StyleLayer.opacity = 30.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
        pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.15, 0.85, False)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.55, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.32, 0.12, 0.85, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.15, 0.80, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
        
    elif style==44:    #Posterize 
        # apply Posterize
        #[G'MIC] fx_posterize 256,100,10,16,64,0,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='256,100,10,16,64,0,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_posterize "+str(parms)+" ")
        StyleLayer.name = "Posterize Layer "+str(parms)
        StyleLayer.opacity = 15.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        Temp = Styled.copy()
        newImage.add_layer (Temp, 0)
        pdb.gimp_drawable_desaturate (Temp, DESATURATE_LUMINANCE)
        Temp.opacity = 50.0
        Styled = pdb.gimp_image_merge_down (newImage, Temp, CLIP_TO_BOTTOM_LAYER)
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
         
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 1.00, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.60, 0.10, 1.00, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.40, 0.10, 1.00, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.10, 1.00, False)
        elif mean<240:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.15, 0.10, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.10, 0.10, 1.00, False)
        '''    
        pdb.gimp_drawable_equalize (Styled, False)
        pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                  1.00, 0.50, 1.00, False)  # only medium-light values
        '''

        if finallayermode==0: autolayermode = LAYER_MODE_PIN_LIGHT  
        
    elif style==45:    #Quadtree Variations
        # apply Quadtree Variations 
        #[G'MIC] fx_quadtree 2,1024,0.5,0,3,1.5,1,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='2,1024,0.5,0,3,1.5,1,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_quadtree "+str(parms)+" ")
        StyleLayer.name = "Quadtree Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
         
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.10, 0.90, False)
        elif mean<175:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.60, 0.10, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.36, 0.12, 0.85, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.25, 0.15, 0.80, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
        
    elif style==46:    #Rodilius
        # apply Rodilius 
        #[G'MIC] fx_rodilius 10,5,300,5,30,0,0,0,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='10,5,300,5,30,0,0,0,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_rodilius "+str(parms)+" ")
        StyleLayer.name = "Rodilius Layer "+str(parms)
        StyleLayer.opacity = 30.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_drawable_levels_stretch (StyleLayer)
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMA)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<170:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.66, 0.00, 1.00, False)
        elif mean<190:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.44, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.00, 1.00, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY    #SCREEN
    
    elif style==47:     #Shapeism
        # apply Shapeism 
        #[G'MIC] fx_shapeism 4,7,0.38,0,1,8,64,8,3,1,5,0.5,1,127,127,127,255
                  
        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='4,7,0.38,0,1,8,64,8,3,1,5,0.5,1,127,127,127,255'
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_shapeism "+str(parms)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Shapeism Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<170:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.66, 0.00, 1.00, False)
        elif mean<190:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.44, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.30, 0.00, 1.00, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
    
    elif style==48:    #Sharp Abstract
        # apply Sharp Abstract 
        #[G'MIC] fx_sharp_abstract 32,16,0.5,0,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='32,16,0.5,0,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_sharp_abstract "+str(parms)+" ")
        StyleLayer.name = "SharpAbstr Layer "+str(parms)
        StyleLayer.opacity = 75.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        
        if mean<150:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<170:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.60, 0.00, 1.00, False)
        elif mean<190:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.36, 0.00, 1.00, False)
        elif mean<210:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.22, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.14, 0.00, 1.00, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY     
    
    elif style==49:     #Simple Noise
        # apply Simple Noise 
        #[G'MIC] fx_SimpleNoiseCanvas 0.7,20,0,5,5,0,255,0,0,0,0,0,1,0,255,255,255,255,0

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='0.7,20,0,5,5,0,255,0,0,0,0,0,1,0,255,255,255,255,0'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_SimpleNoiseCanvas "+str(parms)+" ")
        StyleLayer = pdb.gimp_image_get_active_layer (newImage) # lost ????
        StyleLayer.name = "Simple Noise Layer "+str(parms)
        StyleLayer.opacity = 22.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMINANCE)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_local_processing 0,75,10,50,20,0,7,0,50,50 ")
        #pdb.gimp_display_new (newImage)
        #return
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.66, 0.00, 1.00, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.44, 0.00, 1.00, False)
        elif mean<220:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.28, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                          0.15, 0.00, 0.75, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
        
    elif style==50:     #Skeletik
        # apply Skeletik 
        #[G'MIC] samj_Test_Skeletik 10,1,0,0,0,5,1.25,255,255,255,255

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='10,1,0,0,0,5,1.25,255,255,255,255'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -samj_Test_Skeletik "+str(parms)+" ")
        StyleLayer = pdb.gimp_image_get_active_layer (newImage) # lost ????
        StyleLayer.name = "Skeletik Layer "+str(parms)
        StyleLayer.opacity = 100.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        #pdb.gimp_display_new (newImage)
        #return
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        if finallayermode==0: autolayermode = LAYER_MODE_HSV_VALUE
        
    elif style==51:    #Sketch
        # apply Sketch 
        #[G'MIC] fx_sketchbw 3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,5.8,0,1,4,1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='3,45,180,30,1.75,0.02,0.5,0.75,0.1,0.7,3,5.8,0,1,4,1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_sketchbw "+str(parms)+" ")
        StyleLayer.name = "Sketch Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.25, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.25, 1.0, False,
                                          0.66, 0.00, 1.00, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.25, 1.0, False,
                                          0.44, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.25, 1.0, False,
                                          0.29, 0.00, 1.00, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
        
    elif style==52:    #Smooth abstract
        # apply Smooth Abstract 
        #[G'MIC] fx_smooth_abstract 40,0,20,0.2,40,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='40,0,20,0.2,40,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_smooth_abstract "+str(parms)+" ")
        StyleLayer.name = " Smooth Abstract Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.16, 1.0, False,
                                          1.00, 0.00, 1.00, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.18, 1.0, False,
                                          0.66, 0.00, 1.00, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.20, 1.0, False,
                                          0.44, 0.00, 1.00, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.22, 1.0, False,
                                          0.29, 0.00, 1.00, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
    
    elif style==53:  #Stylize
        # apply Stylize Portrait DeMetzinger (=deafult style 13)
        #[G'MIC] fx_stylize 13,5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0
        
        stylezedname = [
                        "2 Braque  Landscape near Antwerp",
                        "3 Braque  Le Viaduc &agrave; L'Estaque",
                        "4 Braque  Little Bay at La Ciotat",
                        "5 Braque  The Mandola",
                        "6 Christine Garner  Black Colour Pencil",
                        "7 Christine Garner  Colour Pencil Sepia",
                        "8 Christine Garner  Dark Coloured Pencil",
                        "9 Christine Garner  Pencil",
                        "10 Christine Garner  Sketching Pastel",
                        "11 Christine Garner  Willow Charcoal",
                        "12 Delaunay  Windows Open Simultaneously",
                        "13 Delaunay  Portrait De Metzinger",
                        "14 Hokusai  The Great Wave",
                        "15 Kandinsky  Squares with Concentric Circles",
                        "16 Kandinsky  Yellow-Red-Blue",
                        "17 Klee  Death and Fire",
                        "18 Klee  In the Style of Kairouan",
                        "19 Klee  Oriental Pleasure Garden Anagoria",
                        "20 Klee  Polyphony 2",
                        "21 Klee  Red Waistcoat",
                        "22 Klimt  The Kiss",
                        "23 Mondrian  Composition in Red-Yellow-Blue",
                        "24 Mondrian  Evening; Red Tree",
                        "25 Mondrian  Gray Tree",
                        "26 Monet  San Giorgio Maggiore at Dusk",
                        "27 Monet  Water-Lily Pond",
                        "28 Monet  Wheatstacks - End of Summer",
                        "29 Munch  The Scream",
                        "30 Picabia  Udnie",
                        "31 Picasso  Les Demoiselles D'Avignon",
                        "32 Picasso  Seated Woman",
                        "33 Picasso  The Reservoir - Horta De Ebro",
                        "34 Pollock  Convergence",
                        "35 Pollock  Summertime Number 9A",
                        "36 Van Gogh  Almond Blossom",
                        "37 Van Gogh  Irises",
                        "38 Van Gogh  The Starry Night",
                        "39 Van Gogh  Wheat Field with Crows",    
                        ]

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parmsfull='13,5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0'
        # choose a random type (first parm)
        parms1=(random.randint(2,39))    # type of style (first 2 are "customer")
        parmsrest=',5,0,0,0.5,2,3,0.5,0.1,3,3,0,0.7,1,0,1,0,5,5,7,1,30,2,2,1.85,0'  #all the rest
        pdb.gimp_message ("stylize style chosen random="+str(stylezedname[parms1]))
        #parms=str(parms1)+str(parmsrest)
        
        # half size, because very slow
        pdb.gimp_layer_scale (StyleLayer,
                              StyleLayer.width/2,
                              StyleLayer.height/2,
                              True)
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_stylize "+str(parms1)+str(parmsrest)+" ")
        # back to original size
        pdb.gimp_layer_scale (StyleLayer,
                              baseforeffect.width,
                              baseforeffect.height,
                              True)        
        StyleLayer.name = "Stylize Layer "+str(parms1)+str(parmsrest)
        StyleLayer.opacity = 40.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
        pdb.gimp_drawable_equalize (StyleLayer, False)
        pdb.gimp_drawable_desaturate (StyleLayer, DESATURATE_LUMINANCE)
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          1.00, 0.00, 0.90, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.60, 0.00, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.36, 0.00, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.22, 0.00, 0.90, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
    
    elif style==54:     #Vector Painting
        # apply Vector Painting 
        #[G'MIC] fx_vector_painting 9.7,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='9.7,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_vector_painting "+str(parms)+" ")
        StyleLayer.name = "VectorPaint Layer "+str(parms)
        StyleLayer.opacity = 50.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          1.00, 0.00, 0.90, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.60, 0.00, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.36, 0.00, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.22, 0.00, 0.90, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
    
    elif style==55:     #Warhol
        # apply Warhol
        #[G'MIC] warhol 10,10,10,60

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='10,10,10,60'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -warhol "+str(parms)+" ")
        StyleLayer = pdb.gimp_image_get_active_layer (newImage) # lost ????
        StyleLayer.name = "Warhol Layer"+str(parms)
        StyleLayer.opacity = 5.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        Temp = Styled.copy()
        newImage.add_layer (Temp, 0)
        pdb.gimp_drawable_desaturate (Temp, DESATURATE_LUMINANCE)
        Temp.opacity = 75.0
        Styled = pdb.gimp_image_merge_down (newImage, Temp, CLIP_TO_BOTTOM_LAYER)
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          1.00, 0.00, 0.90, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.60, 0.00, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.36, 0.00, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.22, 0.00, 0.90, False)

        if finallayermode==0: autolayermode = LAYER_MODE_PIN_LIGHT
        
    elif style==56:     #Whirl Drawing
        # apply Whirl Drawing 
        #[G'MIC] fx_draw_whirl 1,0,50,50

        StyleLayer = baseforeffect.copy()
        newImage.add_layer (StyleLayer, 0)
        parms='1,0,50,50'
        pdb.plug_in_gmic_qt(newImage,StyleLayer, 1,0,
        "-v - -fx_draw_whirl "+str(parms)+" ")
        StyleLayer.name = "WhirlDraw Layer"+str(parms)
        StyleLayer.opacity = 20.0
        StyleLayer.mode = LAYER_MODE_DIVIDE
              
        Styled = pdb.gimp_layer_new_from_visible (newImage, newImage, "Styled")
        newImage.add_layer (Styled, 0)
        Styled.name = "Styled"
        Styled.opacity = 100.0
        Styled.mode = LAYER_MODE_NORMAL
        
        # Apply Levels
        (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (Styled, HISTOGRAM_LUMINANCE,
                                                        0.0, 1.0)
        #pdb.gimp_message ("mean Styled="+str(mean))
        #pdb.gimp_display_new (newImage)
        #return
        
        if mean<160:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          1.00, 0.00, 0.90, False)
        elif mean<180:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.60, 0.00, 0.90, False)
        elif mean<200:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.36, 0.00, 0.90, False)
        else:
            pdb.gimp_drawable_levels (Styled, HISTOGRAM_VALUE, 0.00, 1.0, False,
                                          0.22, 0.00, 0.90, False)

        if finallayermode==0: autolayermode = LAYER_MODE_OVERLAY
                                                                        
    #pdb.gimp_display_new (newImage)
    #return
       
    #********************************************************************************        
      
    # BG on top
    BGonTop = baseforeffect.copy()
    newImage.add_layer (BGonTop, 0)
    BGonTop.name = "BG on Top "
    BGonTop.mode = LAYER_MODE_LCH_COLOR
    BGonTop.opacity = 50.0
    
    # BG on top 2
    BGonTop2 = baseforeffect.copy()
    newImage.add_layer (BGonTop2, 0)
    BGonTop2.name = "BG on Top (2)"
    BGonTop2.mode = LAYER_MODE_GRAIN_MERGE
    BGonTop2.opacity = 50.0 
    # remove low values, remove high values  
    pdb.gimp_drawable_levels (BGonTop2, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                   1.00, 0.25, 0.75, False)
    # final layer before outcome
    final_layer = pdb.gimp_layer_new_from_visible (newImage, newImage, "Pre_Outcome")
    newImage.add_layer (final_layer, 0)
    final_layer.name = "Preliminary Outcome"
    if finallayermode==0: final_layer.mode = autolayermode
    
    #pdb.gimp_display_new (newImage)
    #return
    
    # final step
    if doFlatten==True:
        outcome = pdb.gimp_image_merge_visible_layers (newImage, CLIP_TO_BOTTOM_LAYER)
        outcome.name = "Outcome" 
    else:    
        outcome = pdb.gimp_layer_new_from_visible (newImage, newImage, "Outcome")
        newImage.add_layer (outcome, 0)
        pdb.gimp_item_set_visible (outcome, False)
    # remove black values 0.0-25.5 and white values 229.5-255.0
    pdb.gimp_drawable_levels (outcome, HISTOGRAM_VALUE, 0.0, 1.0, False,
                              1.00, 0.1, 0.9, False)
    stylename =  ["Anguish","Aurora","Barbouillage",
                  "BCGraffiti","Blockism","Bokeh","Brushify",
                  "Cartoon","ChalkItUp","CircleAbstr",
                  "ColorAbstr","ColorPencil","ComicBook",
                  "Cubism","Cutout","DiffTensors",
                  "Doodle","DreamSmoothing",
                  "Ellipsionism",
                  "FeltPen","FingerPaint","Fractalize",
                  "Ghost","GraphicBoost","GraphicNovel",
                  "HardSketch","HighLBloom","HopePoster","HoughSketch",
                  "IllustrationLook","Kuwahara",
                  "LineArt","Linify","LylePaint",
                  "MakeSquiggly","MorphoPaint",
                  "PaintwBrush","Painting",
                  "PastelArt","PenDrawing",
                  "PhotoIll",
                  "PolyDelauney","PolyEnergy",
                  "PosterEdges","Posterize",
                  "QuadtreeVar","Rodilius",
                  "Shapeism","SharpAbstract",
                  "SimpleNoise","Skeletik",
                  "Sketch","SmoothAbstr","Stylize (random style)",
                  "VectorPaint",
                  "Warhol","WhirlDraw",
                 ]
    if style==53:  #Stylize
        outcome.name = st+oname+" with an Artistic&Divide effect, " 
        outcome.name = outcome.name+" and a stylized style chosen random "+str(stylezedname[parms1])
    else:
        outcome.name = st+oname+" with an Artistic&Divide effect, style "+str(stylename[style])
    # optional user choice for preprocessing
    prepsname = ["",    
                "DCP Dehaze (good for light image",
                "Dynamic Range Increase",
                "Equalize Local Histogram",
                "Local Contrast Enhancement",
                "Local Normalization",
                "Local Processing (Equalize)",
                "Local Processing (Normalize)",
                ]
    
    if prepstep>0:
        outcome.name = outcome.name+" (prep.step choosen="+str(prepsname[prepstep])+"." 
    pdb.gimp_display_new (newImage)

### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/CrazyFilters/"
copyrightYear="2023"
desc="Artistic and Divide"

register(
    "ArtisticDivide",
    desc+whoiam,desc,
    author,author,
    copyrightYear,
    "Artistic and Divide, version 1.1.0 (initial)",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_ADJUSTMENT, "newsize", "Width+Height of New Image"+
         "\n (0=same as input)", 3600, (0, 7200, 100)),
        (PF_OPTION, "effect", "Choose GMIC Artistic effect ", 0,
                     ["Anguish","Aurora","Barbouillage",
                      "BCGraffiti","Blockism","Bokeh","Brushify",
                      "Cartoon","ChalkItUp","CircleAbstr",
                      "ColorAbstr","ColorPencil","ComicBook",
                      "Cubism","Cutout","DiffTensors",
                      "Doodle","DreamSmoothing",
                      "Ellipsionism",
                      "FeltPen","FingerPaint","Fractalize",
                      "Ghost","GraphicBoost","GraphicNovel",
                      "HardSketch","HighLBloom","HopePoster","HoughSketch",
                      "IllustrationLook","Kuwahara",
                      "LineArt","Linify","LylePaint",
                      "MakeSquiggly","MorphoPaint",
                      "PaintwBrush","Painting",
                      "PastelArt","PenDrawing",
                      "PhotoIll",
                      "PolyDelauney","PolyEnergy",
                      "PosterEdges","Posterize",
                      "QuadtreeVar","Rodilius",
                      "Shapeism","SharpAbstract",
                      "SimpleNoise","Skeletik",
                      "Sketch","SmoothAbstr","Stylize",
                      "VectorPaint",
                      "Warhol","WhirlDraw",
                      ]),
        (PF_OPTION, "preproc", "Choose preprocessing step ", 0,
                     ["*none",
                      "DCP Dehaze (good for light images)",
                      "Dynamic Range Increase",
                      "Equalize Local Histogram",
                      "Local Contrast Enhancement",
                      "Local Normalization",
                      "Local Processing (Equalize)",
                      "Local Processing (Normalize)",
                      ]),        
        (PF_TOGGLE, "flatten", "Flatten Image?", False),
    ],
    [],
    ArtisticDivide,
    menu=menu
)

main()
